% MCMC.m
% 


%% START VALUES AND CHAIN PARAMETERS
clear all;

global Zobs;

T_0 = [100, 100];

warning off

usdata;

load hessmat
  
x0 = xmode;  

%% 
N = 575000;          % number of draws for the chain. 
burn_in_para = 0.25; % Cuts x% of the chain to compute statistic
numbersave = 0;      % Number of times to save the chain as progessing
df = 7;              % Degrees of freedom for multivariate t proposal
kappa = 0.30;        % Tuning parameter for proposal density.

% Note: Using the Hessian for proposal density requires running minPosterior.m
%       or loading hessmat.mat
 
c = kappa*inv(hess); % Uses fmincon Hessian
c = chol(c)';

NumDiscrete = length(T_0); % 
maxT = 140;                % Maximum date of structural change;
minT = 70;                 % Minimum date of structural change;
popT = minT:maxT;
upperT = 1;                % maximum number of updated elements in T

volcker_shift = 91;        % Corresponds to 1979Q4 in our sample 

if upperT > NumDiscrete
    disp('Error: The number of dates breaks to be updated in the proposal cannot be higher than the number of breaks')
    return
end

L = length(x0);
pop = 1:L;

%When to save the output
whensave = linspace(2,N,numbersave);


%% DUAL SINGLE-MOVE SAMPLER
T_prop = zeros(N,NumDiscrete); % Proposals

T_prop(1,:) = T_0; 

Prop_xx = zeros(N,length(x0));
Prop_xx(1,:) = x0';

h = waitbar(0,'Computing, please wait...');

T_out  = zeros(N,NumDiscrete);
xx_out = zeros(N,length(x0));

T_out(1,:) = T_prop(1,:);
xx_out(1,:) = x0';
xx_prop = x0';

kk_T = 0;
kk_xx = 0;
reject_rate = 0;
reject_rate_T = 0;


countersave = 1; %for saving chain

% Main Loop
for j=2:N
    
    % T-block
    T_prop(j,:) = T_out(j-1,:);
    blocksizeT = ceil(upperT*rand);
    blockT = sort(randsample(NumDiscrete,blocksizeT));
    T_prop(j,blockT) = sort(randsample(popT,blocksizeT));
    if T_prop(j,1) < volcker_shift 
       alpha = 0;
       ratio = 0;
       reject_rate_T = reject_rate_T + 1;
    else
    denominator = posteriorB(xx_out(j-1,:),T_out(j-1,:));
    numerator = posteriorB(xx_out(j-1,:),T_prop(j,:));
    ratio = exp(-numerator + denominator);  
    alpha = min(ratio,1);
    end
    if isnan(ratio)
       alpha = 0;
    end
    if rand <= alpha
        T_out(j,:)=T_prop(j,:);
        kk_T = kk_T + 1;
    else
        T_out(j,:)=T_out(j-1,:);
    end
    
    acc_T = 100*kk_T/j;
    disp('----Acceptance rate--T----------')
    disp(acc_T)
    disp('--------------------------------')
   
    % xx-block
    xx_prop = xx_out(j-1,:);
    blocksize = ceil(L*rand);
    block = randsample(pop,blocksize);
    xx_prop(1,block) = xx_out(j-1,block)+ (c(block,block)*trnd(df,blocksize,1))'; % multivariate-t with df degrees of freedom
    Prop_xx(j,:) = xx_prop(1,:);
    if min(xx_prop([1:6,8,13:19])) <= 0 || max(xx_prop([8,13:16])) >= 1 
       alpha2 = 0;
       ratio2 = 0;
       reject_rate = reject_rate + 1;
    else
       denominator2 = posteriorB(xx_out(j-1,:),T_out(j,:));
       numerator2 = posteriorB(xx_prop,T_out(j,:));
       ratio2 = exp(-numerator2 + denominator2);
       alpha2 = min(ratio2,1);
    end
    if isnan(ratio2)
       alpha2 = 0;
    end
    if rand <= alpha2
       xx_out(j,:)=xx_prop;
       kk_xx = kk_xx + 1;
    else
        xx_out(j,:) = xx_out(j-1,:);
    end
    
    acc_xx= 100*kk_xx/j;
     disp('----Acceptance rate--xx---------')
     disp(acc_xx)
     disp('---------------------------------')

    waitbar(j/N,h,sprintf('%12.1f',100*kk_xx/j));

    %save the output 
%     if not(isempty(whensave))
%        if j ==floor(whensave(1,countersave))
%           if j == 2
%              from = 1; 
%           else
%              from = floor(whensave(1,countersave-1))+1;
%           end
% %         T_short = T_out(from:j,:);
% %         xx_short = xx_out(from:j,:);
% %         eval(['save mhout' num2str(countersave) ' T_short acc_T xx_short acc_xx']) 
% %         clear T_short xx_short
%           eval(['save mhall' num2str(countersave)])
%           eval(['delete mhall' num2str(countersave-1) '.mat'])
%           countersave = countersave+1;
%        end
%     end
end

delete(h);

%% COMPUTE STATS
compute_mode = 1;
burn_in = ceil(burn_in_para*N) + 1;

T_mode = mode(T_out(burn_in:end,:));
T_mean = mean(T_out(burn_in:end,:));
T_std = std(T_out(burn_in:end,:));


if compute_mode == 1
    xx_mode = zeros(1,L);
    postpriorplots; % needed to compute mode of xx with kernel smoother.
else
    xx_mode = zeros(1,L);
end;

xx_mean = mean(xx_out(burn_in:end,:));

% Calculate probability intervals

xx_sort = sort(xx_out(burn_in:end,:));
N_sort = length(xx_sort); 
ci_low = xx_sort(round(N_sort*0.05),:);
ci_high = xx_sort(round(N_sort*0.95),:);

disp('--------------------------------------------')
disp('---POST DISTRIBUTION:--DATE BREAK-----------')
disp('--------MODE--&-----MEAN--&--------STD-----')
disp([ T_mode', T_mean', T_std'])
disp('--------------------------------------------')
disp('--------------------------------------------')
disp('----Acceptance rate--T----------------------')
disp(100*kk_T/N)
disp('---------STDEV--OF--SHOCKS---------')
disp('------MODE--&-----MEAN--&----MODE (MINPOST)----CI LOW-----------CI HIGH----')
disp([(xx_mode(1:6))', (xx_mean(1:6))', xmode(1:6), (ci_low(1:6))' , (ci_high(1:6))'])
disp('---------DSGE PARAMETERS------------')
disp('------MODE--&-----MEAN--&----MODE (MINPOST)------')
disp([xx_mode(7:end)',xx_mean(7:end)', xmode(7:end), (ci_low(7:end))' , (ci_high(7:end))'])
disp('--------------------------------------------')
disp('----Acceptance rate--xx---------------------')
disp(100*kk_xx/N)
disp('--------------------------------------------')
disp('-Reject rate--constraint violation')
disp(100*reject_rate/N)

%% Save workspace
% eval(['delete mhall' num2str(countersave-1) '.mat'])
% save mhall_B



